@@ -1,11 +1,10 @@
-from django.db import transaction
+from django import forms
+from django.db import transaction, models
 from django.contrib import admin
-from django.contrib.admin import SimpleListFilter
 from django.core.urlresolvers import reverse
-from django.utils.translation import ugettext_lazy as _
+from django.core.exceptions import ImproperlyConfigured
 from avocado.models import DataField, DataConcept, DataCategory, \
     DataConceptField, DataView, DataContext, DataQuery
-from avocado.forms import DataFieldAdminForm
 
 
 class PublishedAdmin(admin.ModelAdmin):
@@ -53,65 +52,52 @@ class PublishedAdmin(admin.ModelAdmin):
     mark_unarchived.short_description = 'Unarchive'
 
 
-class LexiconListFilter(SimpleListFilter):
-    title = _('lexicon')
+class DataFieldAdminForm(forms.ModelForm):
+    def clean_app_name(self):
+        app_name = self.cleaned_data.get('app_name')
+        try:
+            models.get_app(app_name)
+        except ImproperlyConfigured:
+            raise forms.ValidationError(u'The app "{0}" could not be found'
+                                        .format(app_name))
+        return app_name
 
-    parameter_name = 'lexicon'
+    def clean(self):
+        cleaned_data = self.cleaned_data
+        app_name = self.cleaned_data.get('app_name')
+        model_name = cleaned_data.get('model_name')
+        field_name = cleaned_data.get('field_name')
 
-    def lookups(self, request, model_admin):
-        return (
-            ('1', _('Yes')),
-            ('0', _('No')),
-        )
+        model = models.get_model(app_name, model_name)
 
-    def queryset(self, request, queryset):
-        value = self.value()
+        if model is None:
+            del cleaned_data['model_name']
+            msg = u'The model "{0}" could not be found in the app "{1}"' \
+                .format(model_name, app_name)
+            self._errors['model_name'] = self.error_class([msg])
+        elif not model._meta.get_field_by_name(field_name):
+            del cleaned_data['field_name']
+            msg = u'The model "{0}" does not have a field named "{1}"' \
+                .format(model_name, field_name)
+            self._errors['field_name'] = self.error_class([msg])
 
-        if value:
-            subquery = queryset.only('app_name', 'model_name', 'field_name')
-            ids = [x.pk for x in subquery if x.lexicon]
-            if value == '1':
-                queryset = queryset.filter(id__in=ids)
-            else:
-                queryset = queryset.exclude(id__in=ids)
-            return queryset
+        return cleaned_data
 
-
-class ObjectSetListFilter(SimpleListFilter):
-    title = _('objectset')
-
-    parameter_name = 'objectset'
-
-    def lookups(self, request, model_admin):
-        return (
-            ('1', _('Yes')),
-            ('0', _('No')),
-        )
-
-    def queryset(self, request, queryset):
-        value = self.value()
-
-        if value:
-            subquery = queryset.only('app_name', 'model_name', 'field_name')
-            ids = [x.pk for x in subquery if x.objectset]
-            if value == '1':
-                queryset = queryset.filter(id__in=ids)
-            else:
-                queryset = queryset.exclude(id__in=ids)
-            return queryset
+    class Meta(object):
+        model = DataField
 
 
 class DataFieldAdmin(PublishedAdmin):
     form = DataFieldAdminForm
 
-    list_display = ('name', 'published', 'archived', 'internal',
+    list_display = ('name', 'published', 'archived', 'internal', 'type',
                     'orphan_status', 'model_name', 'enumerable',
-                    'is_lexicon', 'is_objectset', 'related_dataconcepts')
+                    'related_dataconcepts')
 
-    list_filter = ('published', 'archived', 'internal', 'model_name',
-                   'enumerable', LexiconListFilter, ObjectSetListFilter)
+    list_filter = ('published', 'archived', 'internal', 'model_name', 'type',
+                   'enumerable')
 
-    list_editable = ('published', 'archived', 'internal', 'enumerable')
+    list_editable = ('published', 'archived', 'internal', 'enumerable', 'type')
 
     search_fields = ('name', 'description', 'keywords')
 
@@ -141,8 +127,8 @@ class DataFieldAdmin(PublishedAdmin):
                            'for programmatic access only.',
         }),
 
-        ('Query Modifiers', {
-            'fields': ('translator', 'enumerable')
+        ('Modifiers', {
+            'fields': ('translator', 'enumerable', 'type')
         }),
 
         ('Data Source', {
@@ -167,14 +153,6 @@ class DataFieldAdmin(PublishedAdmin):
         return 'OK'
     orphan_status.short_description = 'Orphan Status'
 
-    def is_lexicon(self, obj):
-        return obj.lexicon
-    is_lexicon.short_description = 'Is Lexicon'
-
-    def is_objectset(self, obj):
-        return obj.objectset
-    is_objectset.short_description = 'Is ObjectSet'
-
     def related_dataconcepts(self, obj):
         queryset = obj.concepts.only('id', 'name')
         reverse_name = 'admin:avocado_dataconcept_change'
@@ -213,15 +191,16 @@ class DataConceptFieldInlineAdmin(admin.TabularInline):
 
 
 class DataConceptAdmin(PublishedAdmin):
-    list_display = ('name', 'published', 'archived', 'internal', 'category',
-                    'order', 'formatter_name', 'viewable', 'queryable',
-                    'sortable', 'related_datafields')
+    list_display = ('name', 'published', 'archived', 'internal', 'type',
+                    'category', 'order', 'formatter_name', 'viewable',
+                    'queryable', 'sortable', 'related_datafields')
 
-    list_editable = ('published', 'archived', 'internal', 'category', 'order',
-                     'formatter_name', 'viewable', 'queryable', 'sortable')
+    list_editable = ('published', 'archived', 'internal', 'type', 'category',
+                     'order', 'formatter_name', 'viewable', 'queryable',
+                     'sortable')
 
-    list_filter = ('published', 'archived', 'category', 'formatter_name',
-                   'viewable', 'queryable', 'sortable')
+    list_filter = ('published', 'archived', 'category', 'type',
+                   'formatter_name', 'viewable', 'queryable', 'sortable')
 
     inlines = [DataConceptFieldInlineAdmin]
 
@@ -246,7 +225,8 @@ class DataConceptAdmin(PublishedAdmin):
         }),
 
         ('Modifiers', {
-            'fields': ('formatter_name', 'viewable', 'queryable', 'sortable'),
+            'fields': ('formatter_name', 'viewable', 'queryable',
+                       'sortable', 'type'),
         }),
 
         ('Times of Interest', {
@@ -267,9 +247,9 @@ class DataConceptAdmin(PublishedAdmin):
 class DataCategoryAdmin(admin.ModelAdmin):
     model = DataCategory
 
-    list_display = ('name', 'parent', 'order')
+    list_display = ('name', 'parent', 'order', 'published')
 
-    list_editable = ('parent', 'order')
+    list_editable = ('parent', 'order', 'published')
 
 
 class DataViewAdmin(admin.ModelAdmin):
